@@ -1,7 +1,7 @@
 #ifndef __GDICT_APPLET_H_
 #define __GDICT_APPLET_H_
 
-/* $Id: gdict-applet.h,v 1.10 2004/07/19 02:30:42 gman Exp $ */
+/* $Id: gdict-applet.h,v 1.12 2004/11/05 18:02:50 vnoel Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim@cs.cmu.edu>
@@ -20,11 +20,11 @@
  * manipulate the applet should be contained within. */
 typedef struct _GDictApplet {
 	GtkWidget *applet_widget;
-	GtkWidget *button_widget;
+	GtkWidget *image_widget;
 	GtkWidget *box;
 	GtkWidget *entry_widget;
 	GtkWidget *about_dialog;
-	GtkWidget *tooltip_widget;
+	GtkTooltips *tooltips;
 	PanelAppletOrient orient;
 	gboolean handle;
 	gint panel_size;
