@@ -1,4 +1,4 @@
-/* $Id: gdict-defbox.c,v 1.19 2002/04/26 22:28:11 gman Exp $ */
+/* $Id: gdict-defbox.c,v 1.26 2004/11/24 19:35:58 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -18,11 +18,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <string.h>
 #include <ctype.h>
 
-#define GTK_ENABLE_BROKEN
 #include <gnome.h>
 
 #include "gdict-defbox.h"
@@ -64,14 +62,10 @@ static GnomeTextLine *get_line (const char *fontlist, const char *string,
                                 double size);
 #endif
 
-/* gdict_defbox_get_type
- *
- * Register the GDictDefbox type with Gtk's type system if necessary and
- * return the type identifier code
- */
 
 GType 
-gdict_defbox_get_type (void) {
+gdict_defbox_get_type (void)
+{
     static GType gdict_defbox_type = 0;
     
     if (!gdict_defbox_type) {
@@ -95,81 +89,88 @@ gdict_defbox_get_type (void) {
     return gdict_defbox_type;
 }
 
-/* gdict_defbox_init
- *
- * Initialises an instance of a GDictDefbox object
- */
-
 static void 
-gdict_defbox_init (GDictDefbox *defbox) {
+gdict_defbox_init (GDictDefbox *defbox)
+{
     defbox->context = NULL;
     defbox->def_cmd = NULL;
 }
 
-/* gdict_defbox_class_init
- *
- * Initialises a structure describing the GDictDefbox class; sets up signals
- * for defbox events in the Gtk signal management system
- */
-
 static void 
-gdict_defbox_class_init (GDictDefboxClass *class) {
-    GtkObjectClass *object_class;
-    
-    object_class = (GtkObjectClass *) class;
+gdict_defbox_class_init (GDictDefboxClass *class)
+{
+    GObjectClass *object_class;
+
+    object_class = G_OBJECT_CLASS (class);
     
     gdict_defbox_signals[WORD_LOOKUP_START_SIGNAL] =
-        gtk_signal_new ("word_lookup_start", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, word_lookup_start),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("word_lookup_start",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, word_lookup_start),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL] =
-        gtk_signal_new ("word_lookup_done", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, word_lookup_done),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("word_lookup_done",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, word_lookup_done),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[WORD_NOT_FOUND_SIGNAL] =
-        gtk_signal_new ("word_not_found", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, word_not_found),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("word_not_found",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, word_not_found),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[SUBSTR_NOT_FOUND_SIGNAL] =
-        gtk_signal_new ("substr_not_found", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, substr_not_found),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("substr_not_found",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, substr_not_found),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[SOCKET_ERROR_SIGNAL] =
-        gtk_signal_new ("socket_error", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, socket_error),
-                        gtk_marshal_VOID__STRING, GTK_TYPE_NONE, 1,
-                        GTK_TYPE_STRING);
- /*   
-    gtk_object_class_add_signals (object_class, gdict_defbox_signals,
-                                  LAST_SIGNAL);
- */  
+        g_signal_new ("socket_error",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, socket_error),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE,
+                      1,
+                      G_TYPE_STRING);
+
     class->word_lookup_start = NULL;
     class->word_lookup_done = NULL;
     class->word_not_found = NULL;
     class->substr_not_found = NULL;
 }
 
-/* gdict_defbox_new
- *
- * Creates a new GDictDefbox object
- */
-
 GtkWidget *
-gdict_defbox_new (void) {
+gdict_defbox_new (void)
+{
     GDictDefbox *defbox;
-    
-    defbox = GDICT_DEFBOX (gtk_type_new (gdict_defbox_get_type ()));
+
+    defbox = GDICT_DEFBOX (g_object_new (GDICT_TYPE_DEFBOX, NULL));
+
     return GTK_WIDGET (defbox);
 }
 
 /* Setup textview tags
  */
 void
-defbox_setup_tags (GDictDefbox *defbox) {
+defbox_setup_tags (GDictDefbox *defbox)
+{
     GtkTextBuffer *buffer;
 	
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
@@ -182,16 +183,6 @@ defbox_setup_tags (GDictDefbox *defbox) {
 			        "underline", PANGO_UNDERLINE_SINGLE, NULL);
 }
 
-/* gdict_defbox_destroy
- *
- * Deallocates memory associated with the defbox
- */
-
-void
-gdict_defbox_destroy (GDictDefbox *defbox) {
-    g_free (defbox->database);
-}
-
 /* gdict_defbox_lookup
  *
  * Sends the command to the server to commence looking up the definition
@@ -202,19 +193,19 @@ gdict_defbox_destroy (GDictDefbox *defbox) {
  */
 
 gint
-gdict_defbox_lookup (GDictDefbox *defbox, gchar *text) {
+gdict_defbox_lookup (GDictDefbox *defbox, gchar *text)
+{
     g_return_val_if_fail (defbox != NULL, -1);
-    g_return_val_if_fail (IS_GDICT_DEFBOX (defbox), -1);
+    g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), -1);
     g_return_val_if_fail (text != NULL, -1);
     
     while (isspace (*text)) text++;
     
     if (*text == '\0')
         return 0;
-    
-    gtk_signal_emit (GTK_OBJECT (defbox), 
-                     gdict_defbox_signals[WORD_LOOKUP_START_SIGNAL]);
-    
+
+    g_signal_emit (G_OBJECT (defbox), gdict_defbox_signals[WORD_LOOKUP_START_SIGNAL], 0);
+
     gdict_defbox_clear (defbox);
     
     if (defbox->database)
@@ -240,18 +231,18 @@ gdict_defbox_lookup (GDictDefbox *defbox, gchar *text) {
  */
 
 void 
-gdict_defbox_clear (GDictDefbox *defbox) {
+gdict_defbox_clear (GDictDefbox *defbox)
+{
     GtkTextBuffer *buffer;
     GtkTextIter start, end;
-    
+
     g_return_if_fail (defbox != NULL);
-    g_return_if_fail (IS_GDICT_DEFBOX (defbox));
-    
+    g_return_if_fail (GDICT_IS_DEFBOX (defbox));
+
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
-    gtk_text_buffer_get_start_iter (buffer, &start);
-    gtk_text_buffer_get_end_iter (buffer, &end);
+    gtk_text_buffer_get_bounds (buffer, &start, &end);
     gtk_text_buffer_delete (buffer, &start, &end);
-  
+
     if (defbox->def_cmd) {
         dict_command_destroy (defbox->def_cmd);
         defbox->def_cmd = NULL;
@@ -263,24 +254,21 @@ gdict_defbox_clear (GDictDefbox *defbox) {
  * Finds a string of text in the current definition
  */
 
-void 
-gdict_defbox_find (GDictDefbox *defbox, gchar *text, gboolean start) {
+gboolean 
+gdict_defbox_find (GDictDefbox *defbox, const gchar *text, gboolean start)
+{
     GtkTextBuffer *buffer;
     GtkTextIter start_iter, end_iter, iter;
     GtkTextIter match_start, match_end;
     GtkTextMark *mark = NULL;
-    static gchar *lastp = NULL;
-    static gchar *deftext = NULL;
-    gchar *findp;
-    gint i = 0;
 
-    g_return_if_fail (defbox != NULL);
-    g_return_if_fail (IS_GDICT_DEFBOX (defbox));
+    g_return_val_if_fail (defbox != NULL, FALSE);
+    g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), FALSE);
     
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
-    
+
     gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
-    	
+
     if (start) 
     	iter = start_iter;
     else {
@@ -305,11 +293,15 @@ gdict_defbox_find (GDictDefbox *defbox, gchar *text, gboolean start) {
                                gtk_text_buffer_get_mark (buffer, "selection_bound"),
                                &match_start);     
         gtk_text_buffer_create_mark (buffer, "last_search", &match_end, FALSE);
+
+	return TRUE;
     }
     else
-        gtk_signal_emit (GTK_OBJECT (defbox), 
-                         gdict_defbox_signals[SUBSTR_NOT_FOUND_SIGNAL]);
+    {
+        g_signal_emit (G_OBJECT (defbox), gdict_defbox_signals[SUBSTR_NOT_FOUND_SIGNAL], 0);
 
+	return FALSE;
+    }
 }
 
 /* gdict_defbox_reset
@@ -319,7 +311,8 @@ gdict_defbox_find (GDictDefbox *defbox, gchar *text, gboolean start) {
  */
 
 void
-gdict_defbox_reset (GDictDefbox *defbox, dict_context_t *context) {
+gdict_defbox_reset (GDictDefbox *defbox, dict_context_t *context)
+{
     gchar *word;
     
     if (context != defbox->context || 
@@ -346,14 +339,15 @@ gdict_defbox_reset (GDictDefbox *defbox, dict_context_t *context) {
  */
 
 void 
-gdict_defbox_print (GDictDefbox *defbox) {
+gdict_defbox_print (GDictDefbox *defbox)
+{
     GnomePrintContext *pc;
     GList *node;
     dict_res_t *res;
     int x = 32, y = 700;
 
     g_return_if_fail (defbox != NULL);
-    g_return_if_fail (IS_GDICT_DEFBOX (defbox));
+    g_return_if_fail (GDICT_IS_DEFBOX (defbox));
     g_return_if_fail (gdict_printer != NULL);
 
     if (defbox->def_cmd == NULL)
@@ -399,9 +393,10 @@ gdict_defbox_print (GDictDefbox *defbox) {
  */
 
 gchar *
-gdict_defbox_get_word (GDictDefbox *defbox) {
+gdict_defbox_get_word (GDictDefbox *defbox)
+{
     g_return_val_if_fail (defbox != NULL, NULL);
-    g_return_val_if_fail (IS_GDICT_DEFBOX (defbox), NULL);
+    g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), NULL);
     
     return defbox->def_cmd ? defbox->def_cmd->search_term : NULL;
 }
@@ -412,7 +407,8 @@ gdict_defbox_get_word (GDictDefbox *defbox) {
  */
 
 static gboolean 
-is_xref (gchar *str, int len) {
+is_xref (gchar *str, int len)
+{
     gint i;
     for (i = 0;  i < len;  i++)
         if (!isupper(str[i]) && !ispunct(str[i]))
@@ -426,7 +422,8 @@ is_xref (gchar *str, int len) {
  */
 
 static gboolean 
-is_number (gchar *str, int len) {
+is_number (gchar *str, int len)
+{
     gint i;
 
     if (str[len-1] != '.')
@@ -445,7 +442,8 @@ is_number (gchar *str, int len) {
  */
 
 static gboolean 
-is_part (gchar *str, int len) {
+is_part (gchar *str, int len)
+{
     gchar buf[3];
 
     if ((len < 1) || (len > 2))
@@ -464,8 +462,8 @@ is_part (gchar *str, int len) {
 
 static void
 insert_text_with_tags (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *p, gint len,
-		       gboolean bold, gboolean italic, gboolean underline) {
-	
+		       gboolean bold, gboolean italic, gboolean underline)
+{
 	GtkTextMark *mark;
 	GtkTextIter start;
 	
@@ -473,29 +471,46 @@ insert_text_with_tags (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *p, gint
 	gtk_text_buffer_insert (buffer, iter, p, len);
 	gtk_text_buffer_get_iter_at_mark (buffer, &start, mark);
 	gtk_text_buffer_delete_mark (buffer, mark);
+
 	if (bold)
 		gtk_text_buffer_apply_tag_by_name (buffer, "bold", &start, iter);
 	if (italic)
 		gtk_text_buffer_apply_tag_by_name (buffer, "italic", &start, iter);
 	if (underline)
 		gtk_text_buffer_apply_tag_by_name (buffer, "underline", &start, iter);
-			       
 }
 
+static void
+insert_link (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *text, gint len)
+{
+	GtkTextTag *tag;
+
+	tag = gtk_text_buffer_create_tag (buffer, NULL,
+					  "foreground", "blue",
+					  "underline", PANGO_UNDERLINE_SINGLE,
+					  NULL);
+	g_object_set_data (G_OBJECT (tag), "page", text);
+
+	gtk_text_buffer_insert_with_tags (buffer, iter, text, len , tag, NULL);
+}
+
+
 /* defbox_add
  *
  * Adds a definition to the defbox, performing all necessary formatting
  */
 
 static void 
-defbox_add (GDictDefbox *defbox, gchar *def) {
+defbox_add (GDictDefbox *defbox, gchar *def)
+{
     GtkTextBuffer *buffer;
-    GtkTextIter iter, prev_iter;
-    gchar *p;
+    GtkTextIter iter;
+    gchar *p, *q, *text;
     gint len;
     GdkFont *font;
     GdkColor *color;
     gboolean italic, bold, underline;
+    int length = 0;
 
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
     gtk_text_buffer_get_end_iter (buffer, &iter);
@@ -577,11 +592,22 @@ defbox_add (GDictDefbox *defbox, gchar *def) {
         } else if (*p == '{') {
             font = gdict_pref.typefaces[TYPEFACE_EXAMPLE].font;
             color = gdict_pref.typefaces[TYPEFACE_EXAMPLE].color;
-            bold = FALSE;
+            bold = TRUE;
             italic = TRUE;
             underline = FALSE;
-            ++p;
-        } else if ((*p == ']') || (*p == '}')) {
+			q = p;
+			while (*q != '}')
+			{
+				 length++;
+				 q++;
+                        }				
+			len = length;
+			++p;	
+			text = g_strndup (p, len - 1);
+			insert_link (buffer, &iter, text, len - 1);
+			p += length;
+			length = 0;
+        } else if (*p == ']' ) {
             font = gdict_pref.typefaces[TYPEFACE_BODY].font;
             color = gdict_pref.typefaces[TYPEFACE_BODY].color;
             bold = FALSE;
@@ -626,31 +652,28 @@ static void
 def_error_cb (dict_command_t *command, DictStatusCode code,
               gchar *message, gpointer data)
 {
-    GtkObject *defbox;
+    GObject *defbox;
     dict_command_t *sec_cmd;
-    gchar *string;
-    
-    defbox = GTK_OBJECT (data);
-    
+
+    defbox = G_OBJECT (data);
+
     if (code != DICT_SOCKET_ERROR) {
         GtkWidget *dialog;
         
-        string = g_strdup_printf (_("Error invoking query: %s"), message);
-        dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  	 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                  	 "%s", string, NULL); 
+        dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+						     "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+						     _("Error invoking query"), message);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 	
-	g_free (string);
         sec_cmd = dict_disconnect_command_new ();
         dict_command_invoke (sec_cmd, command->context);
         
-        gtk_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL]);
+        g_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL], 0);
     }
     else {
-        gtk_signal_emit (defbox, gdict_defbox_signals[SOCKET_ERROR_SIGNAL],
-                         message);
+        g_signal_emit (defbox, gdict_defbox_signals[SOCKET_ERROR_SIGNAL], 0, message);
     }
 }
 
@@ -660,7 +683,8 @@ def_error_cb (dict_command_t *command, DictStatusCode code,
  */
 
 static void 
-def_data_cb (dict_command_t *command, dict_res_t *res, gpointer data) {
+def_data_cb (dict_command_t *command, dict_res_t *res, gpointer data)
+{
     GDictDefbox *defbox;
     
     defbox = GDICT_DEFBOX (data);
@@ -676,14 +700,14 @@ static void
 def_status_cb (dict_command_t *command, DictStatusCode code, 
                int num_found, gpointer data)
 {
-    GtkObject *defbox;
-    
-    defbox = GTK_OBJECT (data);
-    
+    GObject *defbox;
+
+    defbox = G_OBJECT (data);
+
     if (code == DICT_STATUS_OK)
-      gtk_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL]);
+        g_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL], 0);
     else if (code == DICT_STATUS_NO_MATCH)
-      gtk_signal_emit (defbox, gdict_defbox_signals[WORD_NOT_FOUND_SIGNAL]);
+        g_signal_emit (defbox, gdict_defbox_signals[WORD_NOT_FOUND_SIGNAL], 0);
 }
 
 #ifdef HAVE_GNOME_PRINT
@@ -694,7 +718,8 @@ def_status_cb (dict_command_t *command, DictStatusCode code,
  */
 
 static GnomeTextLine *
-get_line (const char *fontlist, const char *string, double size) {
+get_line (const char *fontlist, const char *string, double size)
+{
     GnomeTextAttrEl attrs[4];
     int n_chars, i;
     GnomeTextLayout *layout;
