@@ -22,14 +22,18 @@
 #  include <config.h>
 #endif
 
-#include <fcntl.h>
-#include <gnome.h>
-#include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <glade/glade.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <errno.h>
+#include <fcntl.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
+#include <libgnome/gnome-help.h>
+#include <libgnomeui/gnome-ui-init.h>
+#include <glade/glade.h>
+
+#ifdef USE_HAL
+#  include <hal/libhal.h>
+#endif
 
 #include "gfloppy.h"
 #include "gfloppy-config.h"
@@ -37,18 +41,24 @@
 
 #define NUM_DEVICES_TO_CHECK 4
 
-extern int errno;
-
 static GFloppy floppy;
 static GladeXML *xml;
 static GtkWidget *toplevel = NULL;
 
+gboolean devfs_mode = FALSE;
 gboolean prefer_mkdosfs_backend;
 
 /* Only needed if more then one device exists */
 static GList *valid_devices = NULL;
 
 
+typedef struct _FloppyStatus
+{
+	GFloppyStatus status;
+	gchar *device_path;
+} FloppyStatus;
+
+
 static GFloppyFormattingMode get_selected_formatting_mode ();
 static void type_option_changed_cb     (GtkOptionMenu   *option_menu,
 					gpointer         data);
@@ -58,18 +68,53 @@ static void set_widget_sensitive_from  (GtkToggleButton *button,
 */
 
 static void
-devices_option_activated (GtkWidget *menu_item, gchar *device)
+show_error_dialog (GtkWidget *parent, gchar *main, gchar *secondary)
+{
+	GtkWidget *dialog;
+	GtkDialogFlags flags;
+	
+	if (parent != NULL)
+		flags = GTK_DIALOG_MODAL;
+	
+	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(parent), flags, 
+						     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+						     "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+						     main, secondary);
+	gtk_window_set_title (GTK_WINDOW (dialog), "");
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+}
+
+static void 
+floppy_status_free (FloppyStatus *floppies, gint num_floppies)
+{
+	gint i;
+
+	for (i = 0; i < num_floppies; i++) {
+		g_free (floppies[i].device_path);
+	}
+
+	g_free (floppies);
+}
+
+static char *
+get_mformat_device (char *device)
+{
+	/* FIXME: what the heck is /dev/fd2 and /dev/fd3 ?? */
+	if (!strcmp (device, "/dev/floppy/1") || !strcmp (device, "/dev/fd1"))
+		return "b:";
+
+	return "a:";
+}
+
+static void
+devices_option_activated (GtkWidget *menu_item, char *device)
 {
 	g_free (floppy.device);
 	g_free (floppy.mdevice);
 
 	floppy.device = g_strdup (device);
-
-	/* FIXME: what the heck is /dev/fd2 and /dev/fd3 ?? */
-	if (strncmp (floppy.device, "/dev/fd1", strlen ("/dev/fd1")) == 0)
-		floppy.mdevice = g_strdup ("b:");
-	else
-		floppy.mdevice = g_strdup ("a:");
+	floppy.mdevice = g_strdup (get_mformat_device (device));
 }
 
 static void
@@ -101,7 +146,57 @@ start_format (void)
 	setup_progress_and_run (&floppy, toplevel);
 }
 
-#include <stdlib.h>
+#ifdef USE_HAL
+
+#define HAL_FLOPPY_KEY   "storage.drive_type"
+#define HAL_FLOPPY_VALUE "floppy"
+
+#define HAL_FLOPPY_DEV_KEY "block.device"
+
+static FloppyStatus *
+get_hal_floppies (gint *num_devices, gint *num_ok, LibHalContext *ctx)
+{
+	gint              i;
+	gchar           **device_list;
+	FloppyStatus *floppies;
+
+	device_list = hal_manager_find_device_string_match (ctx,
+							    HAL_FLOPPY_KEY, 
+							    HAL_FLOPPY_VALUE, 
+							    num_devices); 
+
+	floppies = NULL;
+	
+	*num_ok = 0;
+
+	if (*num_devices != 0) {
+		floppies = g_new0 (FloppyStatus, *num_devices);
+
+		for (i = 0; i < *num_devices; i++) {
+			floppies[i].device_path = 
+				hal_device_get_property_string (ctx, 
+								device_list[i], 
+								HAL_FLOPPY_DEV_KEY);
+
+			floppies[i].status = test_floppy_device (floppies[i].device_path);
+
+			if (floppies[i].status == GFLOPPY_DEVICE_OK)
+				(*num_ok)++;
+		}
+		
+		hal_free_string_array (device_list);
+	} else {
+		*num_devices = 1;
+		
+		floppies = g_new0 (FloppyStatus, 1);
+		
+		floppies[0].status = GFLOPPY_NO_DEVICE;
+	}
+
+	return floppies;
+}
+
+#endif
 
 static void
 init_commands (void)
@@ -126,27 +221,19 @@ init_commands (void)
 	if (!floppy.mke2fs_cmd &&
 	    !floppy.mkdosfs_cmd &&
 	    !floppy.mformat_cmd) {
-		GtkWidget *error_dialog;
-
-		error_dialog = gtk_message_dialog_new (NULL, 0,
-						       GTK_MESSAGE_ERROR,
-						       GTK_BUTTONS_OK,
-						       _("Neither the mke2fs nor the mkdosfs/mformat applications are installed. You can't format a floppy without one of them."));
-
-		gtk_dialog_run (GTK_DIALOG (error_dialog));
-		gtk_widget_destroy (error_dialog);
-
+		show_error_dialog (NULL, _("Cannot Format"),
+				   _("Neither the mke2fs nor the mkdosfs/mformat applications are installed. You can't format a floppy without one of them."));
 		exit (1);
 	}
 }
 
 static void
-init_devices (void)
+init_devices (gpointer ctx)
 {
 	GtkWidget *device_option;
 	GtkWidget *device_entry;
 	GtkWidget *device_label;
-	gchar *msg = NULL;
+	char *msg = NULL;
 	gint ok_devices_present = 0;
 
 	/* First check --device arg */
@@ -168,80 +255,93 @@ init_devices (void)
 		}
 		if (msg) {
 			GtkWidget *toplevel = glade_xml_get_widget (xml, "toplevel");
-			GtkWidget *dialog;
-
-			gtk_widget_set_sensitive (toplevel, FALSE);
-			dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel), 0,
-							 GTK_MESSAGE_ERROR,
-							 GTK_BUTTONS_OK,
-							 msg);
-			gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (dialog);
+			show_error_dialog (toplevel, _("Cannot initialize device"), msg);
 			exit (1);
 		}
 		/* Device is okay */
 		ok_devices_present = 1;
 	} else {
-		GFloppyStatus status[NUM_DEVICES_TO_CHECK];
-		gint i;
-		char device[32];
+		gint num_devices_to_check = NUM_DEVICES_TO_CHECK;
 
-		for (i = 0; i < NUM_DEVICES_TO_CHECK; i++) {
-			sprintf(device,"/dev/fd%d", i);
-			status[i] = test_floppy_device (device);
-			if (status[i] == GFLOPPY_DEVICE_OK)
-				ok_devices_present++;
+		gint i;
+		FloppyStatus *floppies;
+
+#ifdef USE_HAL
+		gint num_hal_floppies_ok;
+
+		if (ctx != NULL) {
+			floppies = get_hal_floppies (&num_devices_to_check, 
+						     &num_hal_floppies_ok, 
+						     (LibHalContext *) ctx);
+
+			ok_devices_present += num_hal_floppies_ok;
+		} 
+		else 
+		/* If hal didn't initialize properly, fall back to the manual way */
+#endif 
+		{
+			floppies = g_new0 (FloppyStatus, num_devices_to_check);
+			
+			for (i = 0; i < num_devices_to_check; i++) {
+				floppies[i].device_path = g_new0 (gchar, 32);
+				sprintf (floppies[i].device_path,"/dev/floppy/%d", i);
+				floppies[i].status = test_floppy_device (floppies[i].device_path);
+				
+				if (floppies[i].status == GFLOPPY_NO_DEVICE && !devfs_mode) {
+					sprintf (floppies[i].device_path,"/dev/fd%d", i);
+					floppies[i].status = test_floppy_device (floppies[i].device_path);
+				} else
+					devfs_mode = TRUE;
+				
+				if (floppies[i].status == GFLOPPY_DEVICE_OK) 
+					ok_devices_present++;
+			}
 		}
+
 		if (ok_devices_present == 0) {
 			GtkWidget *toplevel;
 			GtkWidget *dialog;
 
-			/* Lets assume that there's only a problem with /dev/fd0 */
-			switch (status[0]) {
+			/* FIXME: Let's assume that there's only a problem with /dev/fd0 */
+			switch (floppies[0].status) {
 			case GFLOPPY_NO_DEVICE:
-				msg = g_strdup_printf (_("Unable to open the device %s, formatting cannot continue."), "/dev/fd0");
+				msg = g_strdup_printf (_("Unable to open any device, formatting cannot continue."));
 				break;
 			case GFLOPPY_DEVICE_DISCONNECTED:
-				msg = g_strdup_printf (_("The device %s is disconnected.\nPlease attach device to continue."), "/dev/fd0");
+				msg = g_strdup_printf (_("The device %s is disconnected.\nPlease attach device to continue."), _("/dev/floppy/0 or /dev/fd0"));
 				break;
 			case GFLOPPY_INVALID_PERMISSIONS:
-				msg = g_strdup_printf (_("You do not have the proper permissions to write to %s, formatting will not be possible.\nContact your system administrator about getting write permissions."), "/dev/fd0");
+				msg = g_strdup_printf (_("You do not have the proper permissions to write to %s, formatting will not be possible.\nContact your system administrator about getting write permissions."), _("/dev/floppy/0 or /dev/fd0"));
 				break;
 			default:
 				g_assert_not_reached ();
 			}
+
 			toplevel = glade_xml_get_widget (xml, "toplevel");
 			gtk_widget_set_sensitive (toplevel, FALSE);
-			dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel), 0,
-							 GTK_MESSAGE_ERROR,
-							 GTK_BUTTONS_OK,
-							 msg);
-			gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (dialog);
+			show_error_dialog (toplevel, _("Cannot initialize device"), msg);
 			exit (1);
 		} else if (ok_devices_present == 1) {
-			for (i = 0; i < NUM_DEVICES_TO_CHECK; i++) {
-				if (status[i] == GFLOPPY_DEVICE_OK) {
-					floppy.device = g_strdup_printf ("/dev/fd%d", i);
+			for (i = 0; i < num_devices_to_check; i++)
+				if (floppies[i].status == GFLOPPY_DEVICE_OK) {
+					floppy.device = g_strdup (floppies[i].device_path);
 					break;
 				}
-			}
 		} else {
-			for (i = 0; i < NUM_DEVICES_TO_CHECK; i++) {
-				if (status[i] == GFLOPPY_DEVICE_OK) {
-					valid_devices = g_list_append (valid_devices, g_strdup_printf ("/dev/fd%d", i));
+			for (i = 0; i < num_devices_to_check; i++)
+				if (floppies[i].status == GFLOPPY_DEVICE_OK) {
+					valid_devices = g_list_append (valid_devices, 
+								       g_strdup (floppies[i].device_path));
+
 					if (floppy.device == NULL)
-						floppy.device = g_strdup_printf ("/dev/fd%d", i);
+						floppy.device = g_strdup (floppies[i].device_path);
 				}
-			}
 		}
+
+		floppy_status_free (floppies, num_devices_to_check);
 	}
 
-	/* FIXME: what the heck is /dev/fd2 and /dev/fd3 ?? */
-	if (strncmp (floppy.device, "/dev/fd1", strlen ("/dev/fd1")) == 0)
-		floppy.mdevice = g_strdup ("b:");
-	else
-		floppy.mdevice = g_strdup ("a:");
+	floppy.mdevice = g_strdup (get_mformat_device (floppy.device));
 
 	/* set up the UI */
 	device_option = glade_xml_get_widget (xml, "device_option");
@@ -264,7 +364,7 @@ init_devices (void)
 		menu = gtk_menu_new ();
 		for (list = valid_devices; list; list = list->next) {
 			GtkWidget *menu_item;
-			menu_item = gtk_menu_item_new_with_label ((gchar *)list->data);
+			menu_item = gtk_menu_item_new_with_label ((char *)list->data);
 			gtk_signal_connect (GTK_OBJECT (menu_item), "activate", GTK_SIGNAL_FUNC (devices_option_activated), list->data);
 			gtk_widget_show (menu_item);
 			gtk_menu_append (GTK_MENU (menu), menu_item);
@@ -275,27 +375,6 @@ init_devices (void)
 	}
 }
 
-static void
-set_floppy_extended_device (void)
-{
-	switch (floppy.size) {
-	case 0:
-		floppy.extended_device = g_strdup_printf ("%sH1440", floppy.device);
-		break;
-	case 1:
-		floppy.extended_device = g_strdup_printf ("%sh1200", floppy.device);
-		break;
-	case 2:
-		floppy.extended_device = g_strdup_printf ("%sD720", floppy.device);
-		break;
-	case 3:
-		floppy.extended_device = g_strdup_printf ("%sd360", floppy.device);
-		break;
-	default:
-		g_assert_not_reached ();
-	}
-}
-
 static GFloppyFormattingMode
 get_selected_formatting_mode ()
 {
@@ -390,25 +469,10 @@ on_help_button_clicked (GtkWidget *widget, gpointer user_data)
 {
 	GError *error = NULL;
 
-	gnome_help_display_desktop (NULL, "user-guide",
-				    "wgosnautilus.xml",
-				    "gosnautilus-469", &error);
-
+	gnome_help_display_desktop_on_screen (NULL, "gfloppy", "gfloppy", "usage", 
+					      gtk_widget_get_screen (widget), &error);
 	if (error) {
-		GtkWidget *dialog;
-
-		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel), GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						 _("Could not display help for the "
-						   "floppy formatter.\n"
-						   "%s"),
-						 error->message);
-
-		g_signal_connect (dialog, "response",
-				  G_CALLBACK (gtk_widget_destroy), NULL);
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		gtk_widget_show (dialog);
-
+		show_error_dialog (toplevel, _("Could not display help for the floppy formatter."), error->message);
 		g_error_free (error);
 	}
 }
@@ -424,21 +488,17 @@ on_format_button_clicked (GtkWidget *widget, gpointer user_data)
         density_option = glade_xml_get_widget (xml, "density_option");
 	volume_name_entry = glade_xml_get_widget (xml, "volume_name_entry");
 
-	floppy.volume_name = (gchar *) gtk_entry_get_text (GTK_ENTRY (volume_name_entry));
+	floppy.volume_name = (char *) gtk_entry_get_text (GTK_ENTRY (volume_name_entry));
 
 	if (strstr (floppy.volume_name, " ")) {
-		GtkWidget *dialog;
-
-		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel), GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("The volume name can't contain any blank space."));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-
+		show_error_dialog (toplevel, _("Incorrect volume name"),
+				   _("The volume name can't contain any blank space."));
 		return;
 	}
 
+	if (strlen (floppy.volume_name) == 0)
+		floppy.volume_name = NULL;
+
 	gtk_widget_set_sensitive (toplevel, FALSE);
 
 	/* If we can format in either types */
@@ -450,8 +510,6 @@ on_format_button_clicked (GtkWidget *widget, gpointer user_data)
 	floppy.size = gtk_option_menu_get_history (GTK_OPTION_MENU (density_option));
 	floppy.formatting_mode = get_selected_formatting_mode ();
 
-	set_floppy_extended_device ();
-
         start_format ();
 
         gtk_widget_set_sensitive (toplevel, TRUE);
@@ -460,11 +518,8 @@ on_format_button_clicked (GtkWidget *widget, gpointer user_data)
 int
 main (int argc, char *argv[])
 {
-	const GnomeIconData *icon_data;
-	gchar *icon_path = NULL;
 	GConfClient *client;
 	GFloppyConfig config;
-        GnomeIconTheme *icon_theme;
 	GtkWidget *fs_entry;
 	GtkWidget *type_option;
 	GtkWidget *quick_format_rb;
@@ -474,6 +529,10 @@ main (int argc, char *argv[])
 	GSList *l;
         gint base_size;
 
+
+	/* This is for the libhal context, should we need it */
+	gpointer ctx;
+
 	struct poptOption gfloppy_opts[] = {
 		{"device", '\0', POPT_ARG_STRING, NULL, 0, NULL, NULL},
 		{NULL, '\0', 0, NULL, 0, NULL, NULL}
@@ -493,13 +552,11 @@ main (int argc, char *argv[])
                             GNOME_PARAM_POPT_FLAGS, 0,
                             GNOME_PARAM_APP_DATADIR,DATADIR,NULL);
 
-        icon_theme = gnome_icon_theme_new ();
-        icon_path = gnome_icon_theme_lookup_icon (icon_theme, "gnome-dev-floppy", 24.0,
-                                                              &icon_data, &base_size);
+#ifdef USE_HAL
+	ctx = (gpointer) hal_initialize (NULL, FALSE);
+#endif
 
-        if (icon_path)
-	  gnome_window_icon_set_default_from_file (icon_path);
-	g_free (icon_path);
+	gtk_window_set_default_icon_name ("gnome-dev-floppy");
 
 	init_commands ();
 
@@ -510,7 +567,7 @@ main (int argc, char *argv[])
 	if (xml == NULL)
 		g_error ("Cannot load/find gfloppy2.glade");
 
-	init_devices ();
+	init_devices (ctx);
 
 	/* We get the different widgets */
 	
@@ -579,7 +636,7 @@ main (int argc, char *argv[])
 		GtkWidget *menu;
 		GtkWidget *menuitem;
 		GtkWidget *label;
-		const gchar *text;
+		const char *text;
 
 		/* Well, is there's any proper solution ? */
 		menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (type_option));
@@ -619,6 +676,10 @@ main (int argc, char *argv[])
 
 	g_object_unref (G_OBJECT (client));
 
+#ifdef USE_HAL
+	hal_shutdown (ctx);
+#endif
+
 	return 0;
 }
 
