@@ -19,31 +19,12 @@
 
     ---------------------------------------------------------------------- */
 
-#include <config.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <malloc.h>
-#include <string.h>
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include "logview.h"
 #include "logrtns.h"
-
-/* 
- * -------------------
- * Global variables 
- * -------------------
- */
-
-extern int loginfovisible, calendarvisible;
-extern int zoom_visible;
-extern ConfigData *cfg;
-extern GtkLabel *date_label;
-extern UserPrefsStruct *user_prefs;
-extern GtkWidget *view;
-extern GtkWidget *app;
-
+#include "log_repaint.h"
+#include "misc.h"
 
 /*
  * -------------------
@@ -51,10 +32,6 @@ extern GtkWidget *app;
  * -------------------
  */
 
-Log *loglist[MAX_NUM_LOGS];
-Log *curlog;
-int numlogs, curlognum;
-
 char *month[12] =
 {N_("January"), N_("February"), N_("March"), N_("April"), N_("May"),
  N_("June"), N_("July"), N_("August"), N_("September"), N_("October"),
@@ -67,34 +44,8 @@ enum {
    MESSAGE
 };
 
-/*
- * -------------------
- * Function prototypes
- * -------------------
- */
-
-int InitPages (void);
-int GetLineAtCursor (int y);
-int RepaintCalendar (GtkWidget * widget, GdkEventExpose * event);
-int RepaintLogInfo (void);
-int repaint_zoom (void);
-gboolean log_repaint (void);
-void log_redrawdetail (void);
-void DrawLogLines (Log *current_log);
-void CloseApp (void);
-void UpdateStatusArea (void);
-void change_log (int direction);
-void create_zoom_view (void);
-void close_zoom_view (GtkWidget *widget, gpointer data);
-gboolean handle_log_mouse_button (GtkWidget *view, GdkEventButton * event_key);
-void handle_selection_changed_cb (GtkTreeSelection *selection, gpointer data);
-void handle_row_activation_cb (GtkTreeView *treeview, GtkTreePath *path,
-     GtkTreeViewColumn *arg2, gpointer user_data);
-void save_rows_to_expand (Log *log);
-
-static void iterate_thru_children(GtkTreeView  *tree_view,
-                      GtkTreeModel *tree_model, GtkTreePath  *tree_path,
-                      GtkTreePath  *orig, gint *count, gint depth);
+static void DrawLogLines (LogviewWindow *window, Log *current_log);
+static void UpdateStatusArea (LogviewWindow *window);
 
 /**
  * Recursively called until the row specified by orig is found.
@@ -228,20 +179,25 @@ handle_row_activation_cb (GtkTreeView *treeview, GtkTreePath *path,
     GtkTreeModel *model;
     GtkTreePath *root_tree;
     gint row = 0;
+    LogviewWindow *window = user_data;
 
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+    model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->view));
     root_tree = gtk_tree_path_new_root ();
 
-    iterate_thru_children (GTK_TREE_VIEW (view), model, root_tree, path, &row, 0);
+    iterate_thru_children (GTK_TREE_VIEW (window->view), model, root_tree, path, &row, 0);
     
     gtk_tree_path_free (root_tree);
 
-    curlog->current_line_no = row;
-    create_zoom_view ();
+    window->curlog->current_line_no = row;
 
+    if (window->zoom_visible)
+	    repaint_zoom (window);
+    else {
+	    GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowDetails");
+	    gtk_action_activate (action);
+    }
 }
 
-
 /* ----------------------------------------------------------------------
    NAME:        handle_selection_changed_cb
    DESCRIPTION: User clicked on main window
@@ -250,108 +206,32 @@ handle_row_activation_cb (GtkTreeView *treeview, GtkTreePath *path,
 void
 handle_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 {
-    GtkTreeIter iter;
     GtkTreeModel *model;
-    GtkTreePath *path, *root_tree;
-    gint row = 0;
+    GList *selected_paths, *i;
+    int selected_first = -1, selected_last = -1;
+    LogviewWindow *window = data;
+    GtkTreePath *selected_path;
 
-    if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+    selected_paths = gtk_tree_selection_get_selected_rows (selection, &model);
 
-        path = gtk_tree_model_get_path (model, &iter);
-        root_tree = gtk_tree_path_new_root ();
-
-        curlog->current_path = gtk_tree_path_copy (path);
-        iterate_thru_children (GTK_TREE_VIEW (view), model, root_tree, path, &row, 0);
-
-        gtk_tree_path_free (root_tree);
-        gtk_tree_path_free (path);
-
-        curlog->current_line_no = row;
-        log_redrawdetail ();
+    if (selected_paths) {
+	    for (i = selected_paths; i != NULL; i = g_list_next (i)) {		    
+		    GtkTreePath *root_tree = gtk_tree_path_new_root ();
+		    int row = 0;
+		    selected_path = i->data;
+		    iterate_thru_children (GTK_TREE_VIEW (window->view), model, root_tree, selected_path, &row, 0);		    
+		    if (selected_last == -1 || row > selected_last)
+			    selected_last = row;
+		    if (selected_first == -1 || row < selected_first)
+			    selected_first = row;
+	    }
+	    window->curlog->current_line_no = selected_last;
+	    window->curlog->current_path = gtk_tree_path_copy (selected_path);
+	    window->curlog->selected_line_first = selected_first;
+	    window->curlog->selected_line_last = selected_last;
+	    g_list_foreach (selected_paths, (GFunc) gtk_tree_path_free, NULL);
+	    g_list_free (selected_paths);
     }
-
-}
-
-/* ----------------------------------------------------------------------
-   NAME:        handle_log_mouse_button
-   DESCRIPTION: User clicked on main window: open zoom view. 
-   ---------------------------------------------------------------------- */
-
-gboolean
-handle_log_mouse_button (GtkWidget * win, GdkEventButton *event)
-{
-    if (event->type == GDK_2BUTTON_PRESS) {
-        if (!zoom_visible)
-        create_zoom_view ();
-    }
-
-    return FALSE;
-
-}
-
-/* ----------------------------------------------------------------------
-   NAME:        change_log
-   DESCRIPTION: User switchs to another Log file
-   ---------------------------------------------------------------------- */
-
-void
-change_log (int direction)
-{
-   if (numlogs == 1)
-        return;
-   if (direction > 0) {
-       if (curlognum == numlogs - 1)
-	       curlognum = 0;
-       else
-	       curlognum++;
-   }
-   else {
-       if (curlognum == 0)
-	       curlognum = numlogs - 1;
-       else
-	       curlognum--;
-   }
-
-   save_rows_to_expand (curlog);
-
-   curlog = loglist[curlognum];
-
-   log_repaint ();
-   if (loginfovisible)
-       RepaintLogInfo ();
-   if (calendarvisible)
-       init_calendar_data ();
-   UpdateStatusArea();
-
-}
-
-void 
-save_rows_to_expand (Log *log)
-{
-   GtkTreeModel *tree_model;
-   GtkTreePath *tree_path;
-   GtkTreeIter iter;
-   gint i = 0;
-
-   if (!log->total_lines)
-       return;
-
-   tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (view)); 
-   tree_path = gtk_tree_path_new_root ();
-   gtk_tree_model_get_iter (tree_model, &iter, tree_path);
-
-   do {
-       tree_path = gtk_tree_model_get_path (tree_model, &iter);
-       if (gtk_tree_model_iter_has_child (tree_model, &iter) &&
-           gtk_tree_view_row_expanded (GTK_TREE_VIEW (view), tree_path)) {
-           log->expand_paths[i] = gtk_tree_path_copy (tree_path);
-           ++i; 
-       }
-   }
-   while (gtk_tree_model_iter_next (tree_model, &iter));
-
-   log->expand_paths[i] = NULL;
-
 }
 
 /* ----------------------------------------------------------------------
@@ -360,53 +240,52 @@ save_rows_to_expand (Log *log)
    ---------------------------------------------------------------------- */
 
 gboolean
-log_repaint ()
+log_repaint (LogviewWindow *window)
 {
    GtkTreeModel *tree_model;
 
+   g_return_val_if_fail (LOGVIEW_IS_WINDOW (window), FALSE);
+
    /* Clear the tree view first */
-   tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+   tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->view));
    gtk_tree_store_clear (GTK_TREE_STORE (tree_model));
 
-   log_redrawdetail (); 
-   UpdateStatusArea ();	   
+   if (window->zoom_visible)
+	   repaint_zoom (window);
+   UpdateStatusArea (window);	   
 
    /* Check that there is at least one log */
-   if (curlog == NULL)
-      return FALSE;
+   if (window->curlog == NULL) {
+	   return FALSE;
+   }
    
    /* Draw the tree view */ 
-   DrawLogLines (curlog); 
+   DrawLogLines (window, window->curlog); 
 
    return TRUE;
 
 }
 
 void
-UpdateStatusArea ()
+UpdateStatusArea (LogviewWindow *window)
 {
    struct tm *tdm;
    char status_text[255];
    char *utf8;
    char *buffer;
-   char *window_title;
+   char *statusbar_text;
    /* Translators: Date only format, %x should well do really */
    const char *time_fmt = _("%x"); /* an evil way to avoid warning */
  
-   if (curlog == NULL) { 
-       gtk_label_set_text (date_label, "");    
-       gtk_window_set_title (GTK_WINDOW (app), APP_NAME);
+   if (window->curlog == NULL) { 
+       gtk_statusbar_pop (GTK_STATUSBAR (window->statusbar), 0);
        return;
    }
 
-   if (curlog->name != NULL) {
-       window_title = g_strdup_printf ("%s - %s", curlog->name, APP_NAME);
-       gtk_window_set_title (GTK_WINDOW (app), window_title);
-       g_free (window_title);
-   }
+   logview_set_window_title (window);
 
-   if (curlog->curmark != NULL) {
-       tdm = &curlog->curmark->fulldate;
+   if (window->curlog->curmark != NULL) {
+	   tdm = &(window->curlog)->curmark->fulldate;
 
        if (strftime (status_text, sizeof (status_text), time_fmt, tdm) <= 0) {
    	       /* as a backup print in US style */
@@ -415,14 +294,12 @@ UpdateStatusArea ()
        } else {
            utf8 = LocaleToUTF8 (status_text);
        }
-       gtk_label_get (date_label, (char **)&buffer);
-       /* FIXME: is this if statement needed?  would it make sense 
-       to set the text every time?  doesn't gtk test for it in 
-       a more efficient manner? */
-       if (strcmp (utf8, buffer) != 0)
-	       gtk_label_set_text (date_label, utf8);
-
+       statusbar_text = g_strconcat (_("Last Modified: "), utf8, NULL);
+       gtk_statusbar_pop (GTK_STATUSBAR(window->statusbar), 0);
+       gtk_statusbar_push (GTK_STATUSBAR(window->statusbar), 0, statusbar_text);
+       
        g_free (utf8);
+       g_free (statusbar_text);
    }
 
 }
@@ -467,7 +344,7 @@ GetDateHeader (LogLine *line)
    ---------------------------------------------------------------------- */
 
 void
-DrawLogLines (Log *current_log)
+DrawLogLines (LogviewWindow *window, Log *current_log)
 {
    GtkTreeModel *tree_model;
    GtkTreeIter iter;
@@ -483,7 +360,7 @@ DrawLogLines (Log *current_log)
    if (!current_log->total_lines) 
        return;
 
-   tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+   tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->view));
    cm = cd = -1;
 
    for (i = 0; i < current_log->total_lines; ++i) {
@@ -540,63 +417,29 @@ DrawLogLines (Log *current_log)
 
        /* Expand the rows */
        path = gtk_tree_model_get_path (tree_model, &iter);
-       gtk_tree_view_expand_row (GTK_TREE_VIEW (view), path, FALSE);
+       gtk_tree_view_expand_row (GTK_TREE_VIEW (window->view), path, FALSE);
        current_log->first_time = FALSE;
 
        /* Scroll and set focus on last row */
        path = gtk_tree_model_get_path (tree_model, &child_iter);
-       gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE); 
-       gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL, TRUE, 
+       gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE); 
+       gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (window->view), path, NULL, TRUE, 
           1, 1);
        gtk_tree_path_free (path);
 
    } else {
        /* Expand the rows */
        for (i = 0; current_log->expand_paths[i]; ++i) 
-           gtk_tree_view_expand_row (GTK_TREE_VIEW (view),
+           gtk_tree_view_expand_row (GTK_TREE_VIEW (window->view),
                current_log->expand_paths[i], FALSE);
 			   
        /* Scroll and set focus on the previously focused row */
-       gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), 
+       gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), 
            current_log->current_path, NULL, FALSE); 
-       gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), 
+       gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (window->view), 
            current_log->current_path, NULL, TRUE, 1, 1);
    }
 
    g_free (utf8);
    
 }
-
-/* ----------------------------------------------------------------------
-   NAME:        log_redrawdetail
-   DESCRIPTION: Redraw area were the detailed information is drawn.
-   ---------------------------------------------------------------------- */
-
-void
-log_redrawdetail ()
-{
-  if (zoom_visible)
-    repaint_zoom ();
-
-}
-
-/* ----------------------------------------------------------------------
-   NAME:        InitPages
-   DESCRIPTION: Returns -1 if there was a error otherwise TRUE;
-   ---------------------------------------------------------------------- */
-
-int
-InitPages ()
-{
-   if (user_prefs == NULL || user_prefs->logfile == NULL)
-	return -1;
-
-   curlog = OpenLogFile (user_prefs->logfile);
-   if (curlog == NULL)
-      return -1;
-   curlog->first_time = TRUE;
-   loglist[0] = curlog;
-   curlognum = 0;
-   numlogs++;
-   return TRUE;
-}
