@@ -23,22 +23,11 @@
 #define __LOGVIEW_H__
 
 #include <time.h>
-#include "gtk/gtk.h"
-#include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs.h>
 
-#define LINES_P_PAGE             10
-#define NUM_PAGES                5 
 #define MAX_WIDTH                240
 #define MAX_HOSTNAME_WIDTH       257	/* Need authoritative answer on this value. */
 #define MAX_PROC_WIDTH           60
-#define NUM_LOGS                 2
-#define R_BUF_SIZE               1024	/* Size of read buffer */
-#define MAX_NUM_LOGS             10
-
-#define LOG_CANVAS_H             400
-#define LOG_CANVAS_W             600
-
-#define APP_NAME                 _("System Log Viewer")
 
 /* the following is a simple hashing function that will convert a
  * given date into an integer value that can be used as a key for
@@ -51,19 +40,6 @@
  *    `----------'
  */
 
-/* for the search results */
-enum {
-	LOGVIEW_WINDOW_OUTPUT_WINDOW_NONE,
-	LOGVIEW_WINDOW_OUTPUT_WINDOW_SEARCH,
-};
-
-typedef struct
-{
-  /* Paths ----------------------------------------------------- */
-  char *regexp_db_path, *descript_db_path, *action_db_path;
-
-} ConfigData;
-
 struct __datemark
 {
   time_t time;		/* date           */
@@ -99,28 +75,6 @@ typedef struct
   int level;
 } Description;
 
-typedef struct
-{
-  char *regexp;
-  GList *matching;
-} ProcessDB;
-
-typedef struct
-{
-  char *text;
-  char tag[50];
-} DescriptionEntry;
-  
-typedef struct
-{
-  char tag[50];
-  char *log_name;
-  char *process;
-  char *message;
-  char *description;
-  char *action;
-} Action;
-
 typedef struct
 {
   char message[MAX_WIDTH];
@@ -138,47 +92,35 @@ typedef void (*MonActions)();
 
 typedef struct
 {
-	/* FIXME: This should perhaps be a glist of logfiles to
-	** open on startup. Also one should be able to set the value in
-	** the prefs dialog
-	*/
-	gchar *logfile;
-} UserPrefsStruct;
-
-typedef struct
-{
-  FILE *fp;
-  DateMark *curmark;
-  char name[255];
-  CalendarData *caldata;
-  LogStats lstats;
-  gint current_line_no; /* indicates the line that is selected */
-  gint total_lines; /* no of lines in the file */
-  LogLine **lines; /* actual lines */
-  gboolean first_time;
-  GtkTreePath *current_path;
-  GtkTreePath *expand_paths[32];
-  GHashTable *date_headers; /* stores paths to date headers */
-
-  /* Monitor info */
-  MonActions alert;
-  long offset_end;
-  int mon_numlines;
+	DateMark *curmark;
+	char name[255];
+	char *display_name;
+	CalendarData *caldata;
+	LogStats lstats;
+	gint current_line_no; /* indicates the line that is selected */
+	gint selected_line_first;
+	gint selected_line_last;
+	gint total_lines; /* no of lines in the file */
+	LogLine **lines; /* actual lines */
+	gboolean first_time;
+	GtkTreePath *current_path;
+	GtkTreePath *expand_paths[32];
+	GHashTable *date_headers; /* stores paths to date headers */
+	
+	/* Monitor info */
+	MonActions alert;
+	GnomeVFSFileOffset mon_offset;
+	GnomeVFSMonitorHandle *mon_handle;
+	GnomeVFSHandle *mon_file_handle;
 }
 Log;
 
-
 /*
  *    ,---------------------.
  *    | Function Prototypes |
  *    `---------------------'
  */
 
-ConfigData *CreateConfig(void);
-int IsLeapYear (int year);
-void SetDefaultUserPrefs(UserPrefsStruct *prefs, GConfClient *client);
-int exec_action_in_db (Log *log, LogLine *line, GList *db);
-
 #define LOGVIEW_TYPE_WINDOW		  (logview_window_get_type ())
 #define LOGVIEW_WINDOW(obj)		  (GTK_CHECK_CAST ((obj), LOGVIEW_TYPE_WINDOW, LogviewWindow))
 #define LOGVIEW_WINDOW_CLASS(klass)	  (GTK_CHECK_CLASS_CAST ((klass), LOGVIEW_TYPE_WINDOW, LogviewWindowClass))
@@ -192,10 +134,6 @@ typedef struct _LogviewWindowClass LogviewWindowClass;
 struct _LogviewWindow {
 	GtkWindow parent_instance;
 
-	GConfClient *client;
-
-	gchar *program_name;
-
 	GtkWidget *view;
 	GtkWidget *mon_list_view;
 	GtkWidget *main_view;
@@ -207,35 +145,33 @@ struct _LogviewWindow {
 
 	GtkWidget *zoom_dialog;
 	GtkWidget *zoom_scrolled_window;
+	GtkWidget *zoom_label;
 	GtkListStore *zoom_store;
 	gboolean zoom_visible;
 	
-	GtkWidget *info_dialog;
-	gboolean loginfovisible;
-
 	GtkWidget *calendar_dialog;
 	GtkWidget *calendar;
 	gboolean calendar_visible;
 
+	gboolean loginfovisible;
+
 	GtkWidget *find_bar;
 	GtkWidget *find_entry;
 	GtkToolItem *find_next;
 	GtkToolItem *find_prev;
 	gchar *find_string;
 
-	int numlogs, curlognum;
-	Log *curlog;	
+	Log *curlog;
 
 	gboolean monitored;
 	gint timer_tag;
+	GtkClipboard *clipboard;
 };
 
 struct _LogviewWindowClass {
 	GtkWindowClass parent_class;
 };
 
-GType logview_window_get_type (void);
-GtkWidget *logview_window_new (void);
 void logview_set_window_title (LogviewWindow *window);
 
 #endif /* __LOGVIEW_H__ */
